@@ -32,6 +32,7 @@ from typing import Callable
 import sys
 import os
 import re
+import ast
 import signal
 import argparse
 import locale
@@ -43,8 +44,10 @@ import logging
 import logging.handlers
 
 from gi import require_version # type: ignore
+# pylint: disable=wrong-import-position
 require_version('GLib', '2.0')
 from gi.repository import GLib # type: ignore
+# pylint: enable=wrong-import-position
 
 # set_prgname before importing other modules to show the name in warning
 # messages when import modules are failed. E.g. Gtk.
@@ -52,11 +55,13 @@ GLib.set_application_name('Emoji Picker')
 # This makes gnome-shell load the .desktop file when running under Wayland:
 GLib.set_prgname('emoji-picker')
 
+# pylint: disable=wrong-import-position
 require_version('Gdk', '3.0')
 from gi.repository import Gdk
 require_version('Gtk', '3.0')
 from gi.repository import Gtk
 from gi.repository import GObject
+# pylint: enable=wrong-import-position
 import itb_emoji
 import itb_util
 import itb_pango
@@ -156,9 +161,10 @@ def parse_args() -> Any:
         ('20200701', '13.0'),
         ('20200915', '13.1'),
         ('20210914', '14.0'),
+        ('20220913', '15.0'),
     )
     current_date = time.strftime('%Y%m%d')
-    current_unicode_version = '13.0'
+    current_unicode_version = '15.0'
     for (date, version) in unicode_versions:
         if current_date > date:
             current_unicode_version = version
@@ -226,19 +232,6 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             Gdk.Screen.get_default(),
             style_provider,
             Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-
-        # https://tronche.com/gui/x/icccm/sec-4.html#WM_CLASS
-        # gnome-shell seems to use the first argument of set_wmclass()
-        # to find the .desktop file.  If the .desktop file can be
-        # found, the name shown by gnome-shell in the top bar comes
-        # from that .desktop file and the icon to show is also read
-        # from that .desktop file. If the .desktop file cannot be
-        # found, the second argument of set_wmclass() is shown by
-        # gnome-shell in the top bar.
-        #
-        # It only works like this when gnome-shell runs under Xorg
-        # though, under Wayland things are different.
-        self.set_wmclass('emoji-picker', 'Emoji Picker') # pylint: disable=no-member
         self.set_default_size(700, 400)
         self._modal = modal
         self.set_modal(self._modal)
@@ -303,7 +296,9 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         self._skin_tone_popover: Optional[Gtk.Popover] = None
         self._skin_tone_selected_popover: Optional[Gtk.Popover] = None
 
-        self._main_container = Gtk.VBox()
+        self._main_container = Gtk.Box()
+        self._main_container.set_orientation(Gtk.Orientation.VERTICAL)
+        self._main_container.set_spacing(0)
         self.add(self._main_container) # pylint: disable=no-member
         self._header_bar = Gtk.HeaderBar()
         self._header_bar.set_hexpand(True)
@@ -315,23 +310,23 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         self._main_menu_popover = Gtk.Popover()
         self._main_menu_popover.set_relative_to(self._main_menu_button)
         self._main_menu_popover.set_position(Gtk.PositionType.BOTTOM)
-        self._main_menu_popover_vbox = Gtk.VBox()
+        self._main_menu_popover_vbox = Gtk.Box()
+        self._main_menu_popover_vbox.set_orientation(Gtk.Orientation.VERTICAL)
+        self._main_menu_popover_vbox.set_spacing(0)
         self._main_menu_clear_recently_used_button = Gtk.Button(
             label=_('Clear recently used'))
         self._main_menu_clear_recently_used_button.connect(
             'clicked', self.on_clear_recently_used_button_clicked)
-        self._main_menu_popover_vbox.pack_start(
-            self._main_menu_clear_recently_used_button, False, False, 0)
+        self._main_menu_popover_vbox.add(
+            self._main_menu_clear_recently_used_button)
         if not self._modal:
             self._main_menu_about_button = Gtk.Button(label=_('About'))
             self._main_menu_about_button.connect(
                 'clicked', self.on_about_button_clicked)
-            self._main_menu_popover_vbox.pack_start(
-                self._main_menu_about_button, False, False, 0)
+            self._main_menu_popover_vbox.add(self._main_menu_about_button)
         self._main_menu_quit_button = Gtk.Button(label=_('Quit'))
         self._main_menu_quit_button.connect('clicked', self.on_delete_event)
-        self._main_menu_popover_vbox.pack_start(
-            self._main_menu_quit_button, False, False, 0)
+        self._main_menu_popover_vbox.add(self._main_menu_quit_button)
         self._main_menu_popover.add(self._main_menu_popover_vbox)
         self._main_menu_button.connect(
             'clicked', self.on_main_menu_button_clicked)
@@ -410,7 +405,7 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             'grab-focus', self.on_search_entry_grab_focus)
 
         self._browse_paned = Gtk.HPaned()
-        self._main_container.pack_start(self._browse_paned, True, True, 0)
+        self._main_container.add(self._browse_paned)
         self._browse_paned.set_wide_handle(True)
         self._browse_paned.set_hexpand(True)
         self._browse_paned.set_vexpand(True)
@@ -436,13 +431,18 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         self._long_press_gestures: List[
             Tuple[Any, Tuple[Any, Any, Any, Any]]] = []
 
-        self._left_pane = Gtk.VBox()
+        self._left_pane = Gtk.Box()
+        self._left_pane.set_orientation(Gtk.Orientation.VERTICAL)
+        self._left_pane.set_spacing(0)
         self._left_pane.set_homogeneous(False)
-        self._left_pane.pack_start(self._search_bar, False, False, 0)
-        self._left_pane.pack_start(self._browse_treeview_scroll, True, True, 0)
+        self._left_pane.add(self._search_bar)
+        self._left_pane.add(self._browse_treeview_scroll)
 
-        self._right_pane = Gtk.VBox()
-        self._right_pane.pack_start(self._flowbox_scroll, True, True, 0)
+        self._right_pane = Gtk.Box()
+        self._right_pane.set_orientation(Gtk.Orientation.VERTICAL)
+        self._right_pane.set_spacing(0)
+        self._right_pane.set_homogeneous(False)
+        self._right_pane.add(self._flowbox_scroll)
 
         self._browse_paned.pack1(
             self._left_pane, resize=True, shrink=True)
@@ -612,8 +612,9 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
                  language, label_key, label])
         return True
 
-    def _sort_labels( # pylint: disable=no-self-use
-            self,
+    @classmethod
+    def _sort_labels(
+            cls,
             labels: Dict[str, List[str]],
             language: str) -> List[str]:
         return sorted(
@@ -883,11 +884,11 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
                 with open(self._options_file,
                           mode='r',
                           encoding='UTF-8') as options_file:
-                    options_dict = eval(options_file.read())
+                    options_dict = ast.literal_eval(options_file.read())
             except (PermissionError, SyntaxError, IndentationError) as error:
                 LOGGER.exception('Error when reading options: %s: %s',
                                  error.__class__.__name__, error)
-            except Exception as error:
+            except Exception as error: # pylint: disable=broad-except
                 LOGGER.exception(
                     'Unexpected error when reading options: %s: %s',
                     error.__class__.__name__, error)
@@ -949,11 +950,12 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
                 with open(self._recently_used_emoji_file,
                           mode='r',
                           encoding='UTF-8') as recently_used_file:
-                    recently_used_emoji = eval(recently_used_file.read())
+                    recently_used_emoji = ast.literal_eval(
+                        recently_used_file.read())
             except (PermissionError, SyntaxError, IndentationError) as error:
                 LOGGER.exception('Error reading recently used emoji: %s: %s',
                                  error.__class__.__name__, error)
-            except Exception as error:
+            except Exception as error: # pylint: disable=broad-except
                 LOGGER.exception(
                     'Unexpected error reading recently used emoji: %s: %s',
                     error.__class__.__name__, error)
@@ -1294,6 +1296,9 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             child.set_can_focus(False)
 
         self.show_all() # pylint: disable=no-member
+        if self._flowbox.get_children():
+            event_box = self._flowbox.get_children()[0].get_child()
+            self._emoji_event_box_selected(event_box, popover=False)
         self._candidates_invalid = False
         self._busy_stop()
 
@@ -1351,7 +1356,15 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             # If the text in the search entry changes while
             # the search bar is invisible, ignore it.
             return
-        self._query_string = query_string
+        if query_string.isspace():
+            # Keep it possible to search for space like characters when
+            # typing just white space:
+            self._query_string = query_string
+        else:
+            # If the query_string contains non-whitespace characters,
+            # strip it otherwise 'black cat ' would not be an exact match
+            # for 'black cat'.
+            self._query_string = query_string.strip()
         if self._candidates_invalid:
             if _ARGS.debug:
                 LOGGER.debug(
@@ -1444,7 +1457,8 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             self._emoji_selected_popover = None
         return False
 
-    def _emoji_event_box_selected(self, event_box: Gtk.EventBox) -> bool:
+    def _emoji_event_box_selected(
+            self, event_box: Gtk.EventBox, popover: bool = True) -> bool:
         '''
         Called when an event box containing an emoji
         was selected in the flowbox.
@@ -1470,6 +1484,8 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         self._show_concise_emoji_description_in_header_bar(emoji)
         self._set_clipboards(emoji)
         self._add_to_recently_used(emoji)
+        if not popover:
+            return True # Gdk.EVENT_PROPAGATE is defined as False
         self._emoji_selected_popover = Gtk.Popover()
         self._emoji_selected_popover.set_relative_to(event_box)
         self._emoji_selected_popover.set_position(Gtk.PositionType.TOP)
@@ -1882,7 +1898,8 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             rectangle.width = self._fontsize * 1.5
             rectangle.height = self._fontsize * 1.5
             self._emoji_info_popover.set_pointing_to(rectangle)
-        emoji_info_popover_vbox = Gtk.VBox()
+        emoji_info_popover_vbox = Gtk.Box()
+        emoji_info_popover_vbox.set_orientation(Gtk.Orientation.VERTICAL)
         emoji_info_popover_vbox.set_vexpand(False)
         emoji_info_popover_vbox.set_hexpand(False)
         margin = 0
@@ -1892,12 +1909,11 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         emoji_info_popover_vbox.set_margin_bottom(margin)
         emoji_info_popover_vbox.set_spacing(margin)
         emoji_info_popover_scroll = Gtk.ScrolledWindow()
-        emoji_info_popover_vbox.pack_start(
-            emoji_info_popover_scroll, True, True, 0)
+        emoji_info_popover_vbox.add(emoji_info_popover_scroll)
         emoji_info_popover_listbox = Gtk.ListBox()
         emoji_info_popover_listbox.set_visible(True)
         emoji_info_popover_listbox.set_can_focus(False)
-        emoji_info_popover_listbox.set_vexpand(False)
+        emoji_info_popover_listbox.set_vexpand(True)
         emoji_info_popover_listbox.set_hexpand(False)
         emoji_info_popover_listbox.set_selection_mode(
             Gtk.SelectionMode.NONE)
@@ -1907,6 +1923,7 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         label.set_hexpand(False)
         label.set_vexpand(False)
         label.set_halign(Gtk.Align.START)
+        label.set_valign(Gtk.Align.START)
         label.set_markup(
             f'<span font="{self._font} {self._fontsize * 3}" '
             f'fallback="{str(self._fallback).lower()}">'
@@ -1924,6 +1941,7 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
             label_description.set_hexpand(False)
             label_description.set_vexpand(False)
             label_description.set_halign(Gtk.Align.START)
+            label_description.set_valign(Gtk.Align.START)
             label_description.set_line_wrap(True)
             label_description.set_markup(description)
             emoji_info_popover_listbox.insert(label_description, -1)
@@ -2180,7 +2198,8 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         self._font_popover.set_relative_to(self._font_button)
         self._font_popover.set_position(Gtk.PositionType.BOTTOM)
         self._font_popover.set_vexpand(True)
-        font_popover_vbox = Gtk.VBox()
+        font_popover_vbox = Gtk.Box()
+        font_popover_vbox.set_orientation(Gtk.Orientation.VERTICAL)
         margin = 12
         font_popover_vbox.set_margin_start(margin)
         font_popover_vbox.set_margin_end(margin)
@@ -2191,8 +2210,7 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         font_popover_label.set_text(_('Set Font'))
         font_popover_label.set_visible(True)
         font_popover_label.set_halign(Gtk.Align.FILL)
-        font_popover_vbox.pack_start(
-            font_popover_label, False, False, 0)
+        font_popover_vbox.add(font_popover_label)
         font_popover_search_entry = Gtk.SearchEntry()
         font_popover_search_entry.set_can_focus(True)
         font_popover_search_entry.set_visible(True)
@@ -2201,12 +2219,10 @@ class EmojiPickerUI(Gtk.Window): # type: ignore
         font_popover_search_entry.set_vexpand(False)
         font_popover_search_entry.connect(
             'search_changed', self.on_font_search_entry_search_changed)
-        font_popover_vbox.pack_start(
-            font_popover_search_entry, False, False, 0)
+        font_popover_vbox.add(font_popover_search_entry)
         self._font_popover_scroll = Gtk.ScrolledWindow()
         self._fill_listbox_font('')
-        font_popover_vbox.pack_start(
-            self._font_popover_scroll, True, True, 0)
+        font_popover_vbox.add(self._font_popover_scroll)
         self._font_popover.add(font_popover_vbox)
         if GTK_VERSION >= (3, 22, 0):
             self._font_popover.popup()
